# -*- coding: utf-8 -*-
"""
Aggregation functions♀
"""
import os
import numpy as np
import pandas as pd
import xarray as xr
from nasadap import Nasa, min_max_dates


###################################################
### Parameters

sp_file_name = '{mission}_{product}_v{version:02}'
file_name = '{mission}_{product}_v{version:02}_{from_date}-{to_date}.nc4'

####################################################
### Aggregate files


def year_combine(param_dict, save_dir, username, password, cache_dir, tz_hour_gmt, min_lat, max_lat, min_lon, max_lon, dl_sim_count):
    """
    Function to aggregate the data from the cache to yearly netcdf files and update the cache if new data has been added to the NASA server.

    Parameters
    ----------
    param_dict : dict
        A dict of {mission: {product: dataset}}
    save_dir : str
        The path to where the yearly files should be saved.
    username : str
        The username for the login.
    password : str
        The password for the login.
    cach_dir : str or None
        A path to cache the netcdf files for future reading. If None, the currently working directory is used.
    tz_hour_gmt : int
        The timezone hour from GMT. e.g. GMT+12 would simply be 12.
    min_lat : int, float, or None
        The minimum lat to extract in WGS84 decimal degrees.
    max_lat : int, float, or None
        The maximum lat to extract in WGS84 decimal degrees.
    min_lon : int, float, or None
        The minimum lon to extract in WGS84 decimal degrees.
    max_lon : int, float, or None
        The maximum lon to extract in WGS84 decimal degrees.
    dl_sim_count : int
        The number of simultaneous downloads on a single thread. Speed could be increase with more simultaneous downloads, but up to a limit of the PC's single thread speed.

    Returns
    -------
    None
    """
    time_dict = {'long_name': 'time', 'tz': 'GMT{}'.format(tz_hour_gmt)}

    for m in param_dict:
        print(m)
        ge = Nasa(username, password, m, cache_dir)
        products = param_dict[m]
        for p in products:
            print(p)
            sp_file_name1 = sp_file_name.format(mission=m, product=p, version=7)
            product_path = os.path.join(save_dir, m + '_' + p)
            if not os.path.exists(product_path):
                os.makedirs(product_path)
            files1 = [os.path.join(product_path, f) for f in os.listdir(product_path) if sp_file_name1 in f]
            latest_file = files1[-1]
            print('*Reading existing files...')
            if files1:
                ds1 = xr.open_dataset(latest_file)
                time0 = ds1.time.to_index() - pd.DateOffset(hours=tz_hour_gmt)
                max_date = str(time0.floor('D').max().date())
                max_test_date = ds1.time.max().values
            else:
                ds1 = xr.Dataset()
                min_max = min_max_dates(m, p)
                max_date = str(min_max['start_date'].iloc[0].date())
                max_test_date = np.nan
            print('*Reading new files...')
            ds2 = ge.get_data(p, products[p], from_date=max_date, min_lat=min_lat, max_lat=max_lat, min_lon=min_lon, max_lon=max_lon, dl_sim_count=dl_sim_count)
            ds2['time'] = ds2.time.to_index() + pd.DateOffset(hours=tz_hour_gmt)
            ds2['time'].attrs = time_dict
            if not max_test_date == ds2.time.max().values:
                print('*New data will be added')
                ds3 = ds2.combine_first(ds1)
                ds1.close()
                attr_dict = {key: value for key, value in ds3.attrs.items() if key in ['title']}
                if not 'title' in attr_dict:
                    attr_dict['title'] = ' '.join([m, p])
                attr_dict.update({'ProductionTime': pd.Timestamp.now().isoformat(), 'institution': 'Environment Canterbury', 'source': 'Aggregated from NASA data'})
                ds3.attrs = attr_dict
                all_years = ds3.time.dt.year
                new_years1 = np.unique(ds2.time.dt.year)
                new_years = new_years1[new_years1 >= np.unique(ds1.time.dt.year)[0]]
                print('*Saving new data...')
                for y in new_years:
                    year_index = all_years == y
                    new_ds1 = ds3.sel(time=year_index)
                    new_dates = new_ds1.time.to_index().strftime('%Y%m%d')
                    new_file_name = file_name.format(mission=m, product=p, version=7, from_date=min(new_dates), to_date=max(new_dates))
                    new_file_path = os.path.join(product_path, new_file_name)
                    new_ds1.to_netcdf(new_file_path)
                print('*Removing old file if not the same as new file...')
                if os.path.split(latest_file)[1] != os.path.split(new_file_path)[1]:
                    os.remove(latest_file)

            else:
                print('*No data to be updated')


# -*- coding: utf-8 -*-
"""
Created on Wed Sep 12 10:27:09 2018

@author: MichaelK
"""
import os
import pickle
import pandas as pd
import xarray as xr
import requests
from time import sleep
from lxml import etree
import itertools
from multiprocessing.pool import ThreadPool
#from pydap.client import open_url
from pydap.cas.urs import setup_session
from nasadap.util import min_max_dates, mission_product_dict, master_datasets

#######################################
### Parameters

file_index_name = 'file_index.pickle'

#######################################


def download_files(url, path, session, master_dataset_list, dataset_types, min_lat, max_lat, min_lon, max_lon):
    print('Downloading and saving to...')
    print(path)
#    print(url)
    counter = 4
    while counter > 0:
        try:
            store = xr.backends.PydapDataStore.open(url, session=session)
            ds = xr.open_dataset(store)

            if 'nlon' in ds:
                ds = ds.rename({'nlon': 'lon', 'nlat': 'lat'})
            ds2 = ds[master_dataset_list].sel(lat=slice(min_lat, max_lat), lon=slice(min_lon, max_lon))

            lat = ds2.lat.values
            lon = ds2.lon.values

            for ar in ds2.data_vars:
                ds_date1 = ds.attrs['FileHeader'].split(';\n')
                ds_date2 = dict([t.split('=') for t in ds_date1 if t != ''])
                ds_date = pd.to_datetime(ds_date2['StopGranuleDateTime'])
                da1 = xr.DataArray(ds2[ar].values.reshape(1, len(lon), len(lat)), coords=[[ds_date], lon, lat], dims=['time', 'lon', 'lat'], name=ar)
                da1.attrs = ds2[ar].attrs
                ds2[ar] = da1

            counter = 0
        except Exception as err:
            print(err)
            print('Retrying in 3 seconds...')
            counter = counter - 1
            sleep(3)

    ## Save data as cache
    if not os.path.isfile(path):
#        print('Saving data to...')
#        print(path)
        ds2.to_netcdf(path)

    return ds2[dataset_types]


def parse_dap_xml(date, file_path, mission, product, version, process_level, base_url):
    path1 = file_path.format(mission=mission.upper(), product=product, year=date.year, dayofyear=date.dayofyear, version=version)
    path2 = '/'.join([process_level, path1])
    url1 = '/'.join([base_url, 'opendap', path2, 'catalog.xml'])
    page1 = requests.get(url1)
    et = etree.fromstring(page1.content)
    urls2 = [base_url + c.attrib['ID'] for c in et.getchildren()[2].getchildren()]
    return urls2


class Nasa(object):
    """
    Class to download, select, and convert NASA data via opendap.

    Parameters
    ----------
    username : str
        The username for the login.
    password : str
        The password for the login.
    mission : str
        Mission name.
    cach_dir : str or None
        A path to cache the netcdf files for future reading. If None, the currently working directory is used.

    Returns
    -------
    Nasa object
    """
    missions_products = {m: list(mission_product_dict[m]['products'].keys()) for m in mission_product_dict}


    def __init__(self, username, password, mission, cache_dir=None):
        self.session(username, password, mission, cache_dir)

    def session(self, username, password, mission, cache_dir=None):
        """
        Function to initiate a dap session.

        Parameters
        ----------
        username : str
            The username for the login.
        password : str
            The password for the login.
        mission : str
            Mission name.
        cach_dir : str or None
            A path to cache the netcdf files for future reading. If None, the currently working directory is used.

        Returns
        -------
        Nasa object
        """
        if mission in mission_product_dict:
            self.mission_dict = mission_product_dict[mission]
        else:
            raise ValueError('mission should be one of: ' + ', '.join(mission_product_dict.keys()))

        self.mission = mission

        if isinstance(cache_dir, str):
            if not os.path.exists(cache_dir):
                os.makedirs(cache_dir)
            self.cache_dir = cache_dir
        else:
            self.cache_dir = os.getcwd()

        self.session = setup_session(username, password, check_url='/'.join([self.mission_dict['base_url'], 'opendap',  self.mission_dict['process_level']]))


    def close(self):
        """
        Closes the session.
        """
        self.session.close()


    def get_products(self):
        """
        Function to get the available products for the mission.

        Returns
        -------
        list
        """
        return list(self.mission_dict['products'].keys())



    def get_dataset_types(product):
        """
        Function to get all of the dataset types and associated attributes for a mission.

        Parameters
        ----------
        product : str
            The mission product

        Returns
        -------
        list
        """
        return master_datasets[product]


    def get_data(self, product, dataset_types, from_date=None, to_date=None, min_lat=None, max_lat=None, min_lon=None, max_lon=None, dl_sim_count=30, check_local=True):
        """
        Function to download trmm or gpm data and convert it to an xarray dataset.

        Parameters
        ----------
        product : str
            Data product associated with the mission.
        dataset_types : str or list of str
            The dataset types variable to be extracted.
        from_date : str or None
            The start date that you want data in the format 2000-01-01.
        to_date : str or None
            The end date that you want data in the format 2000-01-01.
        min_lat : int, float, or None
            The minimum lat to extract in WGS84 decimal degrees.
        max_lat : int, float, or None
            The maximum lat to extract in WGS84 decimal degrees.
        min_lon : int, float, or None
            The minimum lon to extract in WGS84 decimal degrees.
        max_lon : int, float, or None
            The maximum lon to extract in WGS84 decimal degrees.
        dl_sim_count : int
            The number of simultaneous downloads on a single thread. Speed could be increase with more simultaneous downloads, but up to a limit of the PC's single thread speed.
        check_local : bool
            Should the local files be checked and read? Pass False if you only want to download files and not check for local files. Any local files will be overwritten!

        Returns
        -------
        xarray dataset
            Coordinates are time, lon, lat
        """
        if product not in self.mission_dict['products']:
            raise ValueError('product must be one of: ' + ', '.join(self.mission_dict['products'].keys()))
        else:
            product_dict = self.mission_dict['products']
            file_path1 = product_dict[product]
        master_dataset_list = master_datasets[product]

        version = self.mission_dict['version']

        if isinstance(dataset_types, str):
            dataset_types = [dataset_types]

        min_max = min_max_dates(self.mission, product)
        min_date = min_max['start_date'][0]
        max_date = min_max['end_date'][0]

        if isinstance(from_date, str):
            from_date = pd.Timestamp(from_date)
            if min_date > from_date:
                from_date = min_date
        else:
            from_date = min_date
        if isinstance(to_date, str):
            to_date = pd.Timestamp(to_date)
            if max_date < to_date:
                to_date = max_date
        else:
            to_date = max_date

        print('Getting data from {start} to {end}'.format(start=from_date, end=to_date))

        dates = pd.date_range(from_date, to_date)

        base_url = self.mission_dict['base_url']

        ## Determine what files are needed
        if 'dayofyear' in file_path1:
            print('Parsing file list from NASA server...')
            file_path = os.path.split(file_path1)[0]
            iter2 = [(date, file_path, self.mission, product, version, self.mission_dict['process_level'], base_url) for date in dates]
            url_list1 = ThreadPool(30).starmap(parse_dap_xml, iter2)
            url_list = list(itertools.chain.from_iterable(url_list1))
        elif 'month' in file_path1:
            print('Generating urls...')
            url_list = ['/'.join([base_url, 'opendap', self.mission_dict['process_level'],  file_path1.format(mission=self.mission.upper(), product=product, year=d.year, month=d.month, date=d.strftime('%Y%m%d'), version=version)]) for d in dates]

        if 'hyrax' in url_list[0]:
            split_text = 'hyrax/'
        else:
            split_text = 'opendap/'
        url_dict = {u: os.path.join(self.cache_dir, os.path.splitext(u.split(split_text)[1])[0].replace('/', '\\') + '.nc4') for u in url_list}
        path_set = set(url_dict.values())

        save_dirs = set([os.path.split(u)[0] for u in url_dict.values()])
        for path in save_dirs:
            if not os.path.exists(path):
                os.makedirs(path)

        ## Find out what files exist locally
        product_dir = file_path1.split('/')[0].format(mission=self.mission.upper(), product=product, version=version)
        product_path = os.path.join(self.cache_dir, self.mission_dict['process_level'], product_dir)
        file_index_path = os.path.join(product_path, file_index_name)

        if os.path.isfile(file_index_path):
            with open(file_index_path, 'rb') as handle:
                master_set = pickle.load(handle)
        else:
            print('Building index of existing local files...')
            master_set = set()
            for path, subdirs, files in os.walk(product_path):
                for name in files:
                    master_set.add(os.path.join(path, name))
            with open(file_index_path, 'wb') as handle:
                pickle.dump(master_set, handle, protocol=pickle.HIGHEST_PROTOCOL)

        ## Load in files locally and remotely
        if check_local:
            print('Checking if files exist locally...')
            local_set = path_set.intersection(master_set)
            remote_dict = {url: local for url, local in url_dict.items() if local not in local_set}
        else:
            local_set = set()
            remote_dict = url_dict.copy()

        ds_list = []
        if local_set:
            print('Reading local files...')
            local_list = list(local_set)
            local_list.sort()
            ds = xr.open_mfdataset(local_list, concat_dim='time', parallel=True)
            ds2 = ds[dataset_types].sel(lat=slice(min_lat, max_lat), lon=slice(min_lon, max_lon))
            ds.close()
            ds_list.append(ds2)

        if remote_dict:
            print('Downloading files from NASA...')

            iter1 = [(u, u0, self.session, master_dataset_list, dataset_types, min_lat, max_lat, min_lon, max_lon) for u, u0 in remote_dict.items()]

            output = ThreadPool(dl_sim_count).starmap(download_files, iter1)

            ds_list.extend(output)

        ds_all = xr.concat(ds_list, dim='time').sortby('time')

        ## Update the file index
        master_set.update(set(remote_dict.values()))
        with open(file_index_path, 'wb') as handle:
            pickle.dump(master_set, handle, protocol=pickle.HIGHEST_PROTOCOL)

        return ds_all




# -*- coding: utf-8 -*-
"""
Utility functions.
"""

import re
import requests
from lxml import etree
from re import search, IGNORECASE, findall
import os
import pandas as pd

###############################################
### Parameters

mission_product_dict = {
        'trmm': {
                'base_url': 'https://disc2.gesdisc.eosdis.nasa.gov:443',
                'process_level': 'TRMM_L3',
                'version': 7,
                'products': {
                        '3B42_Daily': '{mission}_{product}.{version}/{year}/{month:02}/{product}.{date}.{version}.nc4',
                        '3B42': '{mission}_{product}.{version}/{year}/{dayofyear:03}/{product}.{date}.{hour}.{version}.HDF'
                            }
                },
        'gpm': {
                'base_url': 'https://gpm1.gesdisc.eosdis.nasa.gov:443',
                'process_level': 'GPM_L3',
                'version': 5,
                'products': {
                        '3IMERGDE': '{mission}_{product}.{version:02}/{year}/{month:02}/3B-DAY-E.MS.MRG.3IMERG.{date}-S000000-E235959.V{version:02}.nc4',
                        '3IMERGDL': '{mission}_{product}.{version:02}/{year}/{month:02}/3B-DAY-L.MS.MRG.3IMERG.{date}-S000000-E235959.V{version:02}.nc4',
                        '3IMERGDF': '{mission}_{product}.{version:02}/{year}/{month:02}/3B-DAY.MS.MRG.3IMERG.{date}-S000000-E235959.V{version:02}.nc4',
                        '3IMERGHHE': '{mission}_{product}.{version:02}/{year}/{dayofyear:03}/3B-HHR-E.MS.MRG.3IMERG.{date}-S{time_start}-E{time_end}.{minutes}.V{version:02}B.HDF5',
                        '3IMERGHHL': '{mission}_{product}.{version:02}/{year}/{dayofyear:03}/3B-HHR-L.MS.MRG.3IMERG.{date}-S{time_start}-E{time_end}.{minutes}.V{version:02}B.HDF5',
                        '3IMERGHH': '{mission}_{product}.{version:02}/{year}/{dayofyear:03}/3B-HHR.MS.MRG.3IMERG.{date}-S{time_start}-E{time_end}.{minutes}.V{version:02}B.HDF5'
                            }
                }
        }

master_datasets = {'3B42_Daily': ['precipitation', 'randomError', 'HQprecipitation', 'IRprecipitation'],
                   '3B42': ['precipitation', 'relativeError', 'HQprecipitation', 'IRprecipitation'],
                   '3IMERGHHE': ['precipitationQualityIndex', 'IRkalmanFilterWeight', 'precipitationCal', 'HQprecipitation', 'probabilityLiquidPrecipitation', 'randomError', 'IRprecipitation'],
                   '3IMERGHHL': ['precipitationQualityIndex', 'IRkalmanFilterWeight', 'precipitationCal', 'HQprecipitation', 'probabilityLiquidPrecipitation', 'randomError', 'IRprecipitation'],
                   '3IMERGHH': ['precipitationQualityIndex', 'IRkalmanFilterWeight', 'precipitationCal', 'HQprecipitation', 'probabilityLiquidPrecipitation', 'randomError', 'IRprecipitation'],
                   '3IMERGDE': ['precipitationCal', 'HQprecipitation', 'randomError'],
                   '3IMERGDL': ['precipitationCal', 'HQprecipitation', 'randomError'],
                   '3IMERGDF': ['precipitationCal', 'HQprecipitation', 'randomError']}




###############################################
### Functions



def min_max_dates(missions=None, products=None):
    """
    Function to parse the NASA Hyrax dap server via the catalog xml.

    Parameters
    ----------
    missions : str, list of str, or None
        The missions to parse. None will parse all available.
    products : str, list of str, or None
        The products to parse. None will parse all available.

    Returns
    -------
    DataFrame
        indexed by mission and product
    """
    if isinstance(missions, str):
        missions = [missions]
    elif missions is None:
        missions = list(mission_product_dict.keys())
    if isinstance(products, str):
        products = [products]

    min_max_dates = {}

    for m1 in missions:
        base_url = '/'.join([mission_product_dict[m1]['base_url'], 'opendap',  mission_product_dict[m1]['process_level']])
#        print(base_url)
        if products is None:
            product_dict = mission_product_dict[m1]['products']
        else:
            product_dict = {p: mission_product_dict[m1]['products'][p] for p in products if p in mission_product_dict[m1]['products']}
            if not product_dict:
                raise ValueError('No products associated with ' + m1 + ' mission.')

        for p in product_dict:
            print(p)
            file_path = product_dict[p]
            path_split = os.path.split(file_path)
            mission_product = path_split[0].split('/')[0]

            mission_product1 = mission_product.format(mission=m1.upper(), product=p, version=mission_product_dict[m1]['version'])
            base_url2 = '/'.join([base_url, mission_product1, 'catalog.xml'])

            ## Get all years
            page1 = requests.get(base_url2)
            et = etree.fromstring(page1.content)
            dataset = et.getchildren()[2].getchildren()

            min_attrib = dataset[0].attrib
            min_year = min_attrib['name']

            max_attrib = dataset[-1].attrib
            max_year = max_attrib['name']

            ## min date
            min_base_url = '/'.join([base_url, mission_product1, min_year,  'catalog.xml'])
            page1 = requests.get(min_base_url)
            et = etree.fromstring(page1.content)
            dataset = et.getchildren()[2].getchildren()[0]

            min_mon = dataset.attrib['name']

            min_base_url2 = '/'.join([base_url, mission_product1, min_year, min_mon, 'catalog.xml'])

            page1 = requests.get(min_base_url2)
            et = etree.fromstring(page1.content)
            dataset = et.getchildren()[2].getchildren()[0]

            min_file = dataset.attrib['name']
            min_date1 = re.search('.\d\d\d\d\d\d\d\d', min_file).group()[1:]
#            min_time1 = re.search('S\d\d\d\d\d\d', min_file).group()[1:]
#            min_date = pd.to_datetime(min_date1 + ' ' + min_time1, infer_datetime_format=True)
            min_date = pd.to_datetime(min_date1, infer_datetime_format=True)

            ## max date
            max_base_url = '/'.join([base_url, mission_product1, max_year, 'catalog.xml'])
            page1 = requests.get(max_base_url)
            et = etree.fromstring(page1.content)
            dataset = et.getchildren()[2].getchildren()[-1]

            max_mon = dataset.attrib['name']

            max_base_url2 = '/'.join([base_url, mission_product1, max_year,  max_mon, 'catalog.xml'])

            page1 = requests.get(max_base_url2)
            et = etree.fromstring(page1.content)
            dataset = et.getchildren()[2].getchildren()[-1]

            max_file = dataset.attrib['name']
            max_date1 = re.search('.\d\d\d\d\d\d\d\d', max_file).group()[1:]
#            max_time1 = re.search('E\d\d\d\d\d\d', max_file).group()[1:]
#            max_date = pd.to_datetime(max_date1 + ' ' + max_time1, infer_datetime_format=True)
            max_date = pd.to_datetime(max_date1, infer_datetime_format=True)

            ## combine
            min_max_dates.update({p: [m1, min_date, max_date]})

    ## Convert to dataframe
    min_max = pd.DataFrame.from_dict(min_max_dates, orient='index').reset_index()
    min_max.columns = ['product', 'mission', 'start_date', 'end_date']
    min_max1 = min_max.set_index(['mission', 'product'])

    return min_max1


def rd_dir(data_dir, ext):
    """
    Function to read a directory of files and create a list of files associated with a spcific file extension. Can also create a list of file numbers from within the file list (e.g. if each file is a station number.)
    """

    files = [filename for filename in os.listdir(data_dir) if search('.' + ext + '$', filename, IGNORECASE)]

    return files























# -*- coding: utf-8 -*-
"""
Created on Wed May  9 15:12:14 2018

@author: MichaelEK
"""
import pytest
import xarray as xr
from nasadap import Nasa, min_max_dates
from time import time

###############################
### Parameters

username = '' # Need to change for test
password = '' # Need to change for test
mission1 = 'trmm'
mission2 = 'gpm'
product1a = '3B42_Daily'
product1b = '3B42'
product2a = '3IMERGDE'
product2b = '3IMERGDL'
product2c = '3IMERGDF'
product2d = '3IMERGHHE'
product2e = '3IMERGHHL'
product2f = '3IMERGHH'
from_date = '2018-01-19'
to_date = '2018-01-21'
dataset_type1 = 'precipitation'
dataset_type2 = 'precipitationCal'
min_lat=-49
max_lat=-33
min_lon=165
max_lon=180
cache_dir = r'\\fs02\GroundWaterMetData$\nasa\cache\nz'

###############################
### Tests

## trmm

def test_trmm_session():
    ge = Nasa(username, password, mission1, cache_dir)

    assert ge is not None

ge = Nasa(username, password, mission1, cache_dir)


def test_trmm_dataset_types():
    ds_types = ge.get_dataset_types()

    assert ds_types is not None


def test_trmm1():
    ds1 = ge.get_data(product1a, dataset_type1, from_date, to_date, min_lat, max_lat, min_lon, max_lon)

    assert ds1[dataset_type1].shape == (4, 60, 64)


def test_trmm2():
    ds1 = ge.get_data(product1b, dataset_type1, from_date, to_date, min_lat, max_lat, min_lon, max_lon)

    assert ds1[dataset_type1].shape == (32, 60, 64)

ge.close()

## gpm


def test_gpm_session():
    ge = Nasa(username, password, mission2, cache_dir)

    assert ge is not None

ge = Nasa(username, password, mission2, cache_dir)


def test_gpm_dataset_types():
    ds_types = ge.get_dataset_types()

    assert ds_types is not None


def test_gpm1():
    ds2 = ge.get_data(product2a, dataset_type2, from_date, to_date, min_lat, max_lat, min_lon, max_lon)

    assert ds2[dataset_type2].shape == (4, 150, 160)


def test_gpm2():
    ds2 = ge.get_data(product2b, dataset_type2, from_date, to_date, min_lat, max_lat, min_lon, max_lon)

    assert ds2[dataset_type2].shape == (4, 150, 160)


def test_gpm3():
    ds2 = ge.get_data(product2c, dataset_type2, from_date, to_date, min_lat, max_lat, min_lon, max_lon)

    assert ds2[dataset_type2].shape == (4, 150, 160)


def test_gpm4():
    ds2 = ge.get_data(product2d, dataset_type2, from_date, to_date, min_lat, max_lat, min_lon, max_lon)

    assert ds2[dataset_type2].shape == (192, 150, 160)


def test_gpm5():
    ds2 = ge.get_data(product2e, dataset_type2, from_date, to_date, min_lat, max_lat, min_lon, max_lon)

    assert ds2[dataset_type2].shape == (192, 150, 160)


def test_gpm6():
    ds2 = ge.get_data(product2f, dataset_type2, from_date, to_date, min_lat, max_lat, min_lon, max_lon)

    assert ds2[dataset_type2].shape == (192, 150, 160)

ge.close()

#################################
### Other

#hdf1 = 'https://disc2.gesdisc.eosdis.nasa.gov:443/opendap/TRMM_L3/TRMM_3B42.7/1998/002/3B42.19980102.03.7.HDF'
#nc1 = 'https://disc2.gesdisc.eosdis.nasa.gov:443/opendap/TRMM_L3/TRMM_3B42_Daily.7/1998/01/3B42_Daily.19980101.7.nc4'
#nc2 = 'https://gpm1.gesdisc.eosdis.nasa.gov/opendap/hyrax/GPM_L3/GPM_3IMERGDF.05/2014/03/3B-DAY.MS.MRG.3IMERG.20140312-S000000-E235959.V05.nc4'
#hdf2 = 'https://gpm1.gesdisc.eosdis.nasa.gov/opendap/hyrax/GPM_L3/GPM_3IMERGHH.05/2014/071/3B-HHR.MS.MRG.3IMERG.20140312-S000000-E002959.0000.V05B.HDF5'
#
#store = xr.backends.PydapDataStore.open(hdf1, session=ge.session)
#ds = xr.open_dataset(store)
#
#t1 = ds.attrs['FileHeader'].split(';\n')
#t2 = dict([t.split('=') for t in t1 if t != ''])

#ds3 = xr.Dataset(coords={'time': [], 'lat': [], 'lon': []})
#ds3 = xr.Dataset()
#
#ds3.to_netcdf(t1, unlimited_dims='time')
#
#ds3 = ds2.copy()
#
#ds3['time'] = ds3.time.to_series() + pd.DateOffset(days=2)
#
#with xr.open_mfdataset(t1) as ds:
#    print(ds)
##    ds4 = xr.concat([ds2, ds], dim='time')
#    ds4 = ds.combine_first(ds2)
#    ds5 = ds4.combine_first(ds3)
##    ds4 = xr.merge([ds, ds2])
##    ds5 = xr.merge([ds4, ds2])
#    print(ds5)
#
#ds4.to_netcdf(t1, mode='a', unlimited_dims='time')
#ds5.to_netcdf(t1, mode='a', unlimited_dims='time')

ge = Nasa(username, password, mission1, cache_dir)

start1 = time()
ds1 = ge.get_data(product1a, dataset_type1, from_date, to_date, min_lat, max_lat, min_lon, max_lon, dl_sim_count=65)
end1 = time()

diff1 = end1 - start1





